import unittest

from metapub import PubMedFetcher 
from metapub.pubmedfetcher import parse_related_pmids_result
from metapub.pubmedcentral import *

class TestPubmedFetcher(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_article_by_pmid(self):
        pmid = '4'
        fetch = PubMedFetcher()
        article = fetch.article_by_pmid(pmid)
        assert str(article.pmid) == pmid

        pmid = '25763451'
        fetch = PubMedFetcher()
        article = fetch.article_by_pmid(pmid)
        assert str(article.pmid) == pmid

    def test_related_pmids(self):
        ''' * pubmed    (all related links)
            * citedin   (papers that cited this paper)
            * five      (the "five" that pubmed displays as the top related results)
            * reviews   (review papers that cite this paper)
            * combined  (?)
        '''

        expected_keys = ['pubmed', 'citedin', 'five', 'reviews', 'combined']
        xmlstr = open('tests/data/sample_related_pmids_result.xml').read()
        resd = parse_related_pmids_result(xmlstr)
        for key in resd.keys():
            assert key in expected_keys
        assert len(resd['citedin']) == 6
        




import unittest
from hamcrest import assert_that, is_

from metapub.utils import parameterize

class TestUtils(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_parameterize(self):
        j = 'Muscle & Nerve'
        j_param = parameterize(j)
        assert_that(j_param, is_("Muscle+Nerve"))



from lxml.etree import XMLSyntaxError

class MetaPubError(Exception):
    pass

class InvalidPMID(MetaPubError):
    pass

class CrossRefConnectionError(MetaPubError):
    pass

class NoPDFLink(MetaPubError):
    pass

class AccessDenied(MetaPubError):
    pass




from __future__ import absolute_import, print_function

"""metapub.pubmedarticle -- PubMedArticle class instantiated by supplying ncbi XML string."""

import logging
from datetime import datetime
from collections import OrderedDict

from .base import MetaPubObject
from .exceptions import MetaPubError

class PubMedArticle(MetaPubObject):
    '''This PubMedArticle class receives an XML string as its required argument
    and parses it into its constituent parts, exposing them as attributes. 

    Usage:
        paper = PubMedArticle(xml_string)

    To query services to return an article by pmid, use PubMedFetcher, which
    returns PubMedArticle objects.

    When xmlstr is parsed, the `pubmed_type` attribute will be set to one of 'article' or 'book',
    depending on whether PubmedBookArticle or PubmedArticle headings are found in the supplied
    xmlstr at instantiation.

    Since this class needs to work seamlessly in production whether it's a book 
    or an article, the PubmedArticle attributes will always be available (set to None in many
    cases for PubmedBookArticle, e.g. volume, issue, journal), but PubmedBookArticle
    attributes will only be set when pubmed_type='book'.

    PubMedBook special handling of certain attributes:
        * abstract: a joined string from self.book_abstracts
        * title: comes from ArticleTitle

    Special attributes for PubmedBookArticle (pubmed_type='book'):
        * book_id (default: None) - string from IdType="bookaccession", e.g. "NBK1403"
        * book_title (default: None) - string with name of book (as differentiated from ArticleTitle)
        * book_publisher (default: None) - dict containing {'name': string, 'location': string}
        * book_sections (default: []) - dict with key->value pairs as section_name->SectionTitle 
        * book_contribution_date (default: None) - python datetime date
        * book_date_revised (default: None) - python datetime date
        * book_history (default: [])  - dictionary with key->value pairs as PubStatus -> python datetime
        * book_language (default: None) - string (e.g. "eng")
        * book_editors (default: []) - list containing names from 'editors' AuthorList
        * book_abstracts (default: []) - dict with key->value pairs as Label->AbstractText.text)
        * book_medium (default: None) - string (e.g. "Internet")
        * book_synonyms (default: None) - list of disease synonyms (applicable to "gene" book)
        * book_publication_status (default: None) - string (e.g. "ppublish")
    '''

    def __init__(self, xmlstr, *args, **kwargs):
        if xmlstr.find('<PubmedBookArticle>') > -1:
            super(PubMedArticle, self).__init__(xmlstr, 'PubmedBookArticle', args, kwargs)
            self.pubmed_type = 'book'
            self._root = 'BookDocument'
        elif xmlstr.find('<PubmedArticle>') > -1:
            super(PubMedArticle, self).__init__(xmlstr, 'PubmedArticle', args, kwargs)
            self.pubmed_type = 'article'
            self._root = 'MedlineCitation'
        else:
            # assume MedlineCitation is the root XML element
            super(PubMedArticle, self).__init__(xmlstr, None, args, kwargs)
            self.pubmed_type = 'article'
            self._root = '.'

        pmt = self.pubmed_type
        
        # shared between book and article types:
        self.pmid = self._get_pmid()
        self.url  = self._get_url()
        self.authors = self._get_authors() if pmt=='article' else self._get_book_authors()
        self.title = self._get_title() if pmt=='article' else self._get_book_articletitle()
        self.authors_str = self._get_authors_str()
        self.author1_last_fm = self._get_author1_last_fm()        
        self.author1_lastfm = self._get_author1_lastfm()
    
        # 'article' only (not shared):
        self.pages = None if pmt=='book' else self._get_pages()
        self.first_page = None if pmt=='book' else self._get_first_page()
        self.last_page = None if pmt=='book' else self._get_last_page()
        self.volume = None if pmt=='book' else self._get_volume()
        self.issue = None if pmt=='book' else self._get_issue()
        self.volume_issue = None if pmt=='book' else self._get_volume_issue()
        self.doi = None if pmt=='book' else self._get_doi()
        self.pii = None if pmt=='book' else self._get_pii()
        self.pmc = None if pmt=='book' else self._get_pmc()
        self.issn = None if pmt=='book' else self._get_issn()

        #MeSH headings ('article' only)
        self.mesh = self._get_mesh_headings()

        #Chemical associations ('article' only)
        self.chemicals = self._get_chemicals()

        #Grant information (?? 'article' only ??)
        self.grants = self._get_grantlist()

        #Publication Types (?? 'article' only ??)
        self.publication_types = self._get_publication_types()

        # 'book' only:
        self.book_accession_id = None if pmt=='article' else self._get_bookaccession_id()
        self.book_title = None if pmt=='article' else self._get_book_title()
        self.book_publisher = None if pmt=='article' else self._get_book_publisher()
        self.book_language = None if pmt=='article' else self._get_book_language()
        self.book_editors = None if pmt=='article' else self._get_book_editors()
        self.book_abstracts = None if pmt=='article' else self._get_book_abstracts()
        self.book_sections = None if pmt=='article' else self._get_book_sections()
        self.book_copyright = None if pmt=='article' else self._get_book_copyright()
        self.book_medium = None if pmt=='article' else self._get_book_medium()
        self.book_synonyms = None if pmt=='article' else self._get_book_synonyms()
        self.book_publication_status = None if pmt=='article' else self._get_book_publication_status()
        self.book_history = None if pmt=='article' else self._get_book_history()
        self.book_contribution_date = None if pmt=='article' else self._get_book_contribution_date()
        self.book_date_revised = None if pmt=='article' else self._get_book_contribution_date()

        # the shared oddballs, must be done last.
        self.abstract = self._get_abstract() if pmt=='article' else self._get_book_abstract()
        self.journal = self.book_title if pmt=='book' else self._get_journal()
        self.year = self._get_book_year() if pmt=='book' else self._get_year()


    def to_dict(self):
        outd = self.__dict__
        outd.pop('content')
        outd.pop('xmlstr')
        outd.pop('_root')
        return self.__dict__

    def _construct_datetime(self, d):
        names = ['Year', 'Month', 'Day']
        parts = {}
        for name in names:
            if d.find(name) is not None:
                parts[name.lower()] = d.find(name).text
        if 'day' in parts.keys() and 'month' in parts.keys():
            return datetime.strptime('{year}/{month}/{day}'.format(**parts), '%Y/%m/%d').date()
        elif 'month' in parts.keys() and 'year' in parts.keys():
            return datetime.strptime('{year}/{month}'.format(**parts), '%Y/%m').date()
        else:
            return datetime.strptime('{year}'.format(**parts), '%Y').date()

    def _get_bookaccession_id(self):
        for item in self.content.findall('BookDocument/ArticleIdList/ArticleId'):
            if item.get('IdType')=='bookaccession':
                return item.text

    def _get_book_title(self):
        return self._get('BookDocument/Book/BookTitle')

    def _get_book_articletitle(self):
        return self._get('BookDocument/ArticleTitle')

    def _get_book_authors(self):
        authors = [ _au_to_last_fm(au) for au in self.content.findall('BookDocument/AuthorList/Author') ]
        return authors

    def _get_book_publisher(self):
        return self._get('BookDocument/Book/Publisher/PublisherName')

    def _get_book_publisher_location(self):
        return self._get('BookDocument/Book/Publisher/PublisherLocation')

    def _get_book_language(self):
        return self._get('BookDocument/Language')

    def _get_book_editors(self):
        return [ _au_to_last_fm(au) for au in self.content.findall('BookDocument/Book/AuthorList/Author') ]

    def _get_book_abstracts(self):
        abd = OrderedDict()
        for item in self.content.findall('BookDocument/Abstract/AbstractText'):
            abd[item.get('Label')] = item.text
        return abd

    def _get_book_sections(self):
        sections = {}
        for item in self.content.findall('BookDocument/Sections/Section'):
            sec_title = item.find('SectionTitle')
            sections[sec_title.get('sec')] = sec_title.text
        return sections

    def _get_book_abstract(self):
        abstract_strs = ['%s: %s' % (k,v) for k,v in self.book_abstracts.items()]
        return '\n'.join(abstract_strs)

    def _get_book_copyright(self):
        return self._get('BookDocument/Abstract/CopyrightInformation')

    def _get_book_medium(self):
        return self._get('BookDocument/Book/Medium')

    def _get_book_contribution_date(self):
        contribution_date_element = self.content.find('BookDocument/ContributionDate')
        if contribution_date_element is not None:
            return self._construct_datetime(self.content.find('BookDocument/ContributionDate'))
        return None

    def _get_book_date_revised(self):
        return self._construct_datetime(self.content.find('BookDocument/DateRevised'))

    def _get_book_synonyms(self):
        syn_list = self.content.find('BookDocument/ItemList')
        if syn_list is not None and syn_list.get('ListType')=='Synonyms':
            return [item.text for item in self.content.findall('BookDocument/ItemList/Item')]
        else:
            return []

    def _get_book_history(self):
        history = {}
        items = self.content.findall('PubmedBookData/History/PubMedPubDate')
        for item in items:
            history[item.get('PubStatus')] = self._construct_datetime(item)
        return history

    def _get_book_publication_status(self):
        return self._get('PubmedBookData/PublicationStatus')

    def _get_book_year(self):
        if self.book_contribution_date:
            return self.book_contribution_date.year
        return None

    def _get_pmid(self):
        return self._get(self._root+'/PMID')

    def _get_url(self):
        return 'http://ncbi.nlm.nih.gov/pubmed/'+str(self.pmid)

    def _get_abstract(self):
        abstracts = self.content.findall(self._root + '/Article/Abstract/AbstractText')
        if abstracts == []:
            return self._get(self._root+'/Article/Abstract/AbstractText')

        if len(abstracts)==1:
            return abstracts[0].text

        # this is a type of PMA with several AbstractText listings (like a Book)
        abd = OrderedDict()
        for ab in abstracts:
            abd[ab.get('Label')] = ab.text
        return '\n'.join(['%s: %s' % (k,v) for k,v in abd.items()])

    def _get_authors(self):
        # N.B. Citations may have 0 authors. e.g., pmid:7550356
        authors = [ _au_to_last_fm(au) for au in self.content.findall(self._root+'/Article/AuthorList/Author') ]
        return authors

    def _get_authors_str(self):
        return '; '.join(self.authors) 

    def _get_author1_last_fm(self):
        '''return first author's name, in format Last INITS (space between surname and inits)'''
        #return _au_to_last_fm(self.content.find(self._root+'/Article/AuthorList/Author'))
        if self.authors:
            return self.authors[0]
        else:
            return None

    def _get_author1_lastfm(self):
        '''return first author's name, in format LastINITS'''
        if self.author1_last_fm is not None:
            return self.author1_last_fm.replace(' ','')
        return None

    def _get_journal(self):
        j = self._get(self._root+'/Article/Journal/ISOAbbreviation')
        if j is None:
            # e.g., http://www.ncbi.nlm.nih.gov/pubmed?term=21242195
            j = self._get(self._root+'/Article/Journal/Title')
        return j

    def _get_pages(self):
        return self._get(self._root+'/Article/Pagination/MedlinePgn') 

    def _get_first_page(self):
        try:
            return self.pages.split('-')[0]
        except AttributeError:
            return self.pages

    def _get_last_page(self):
        #TODO: return true last page in situations like self.pages = "148-52"
        #           i.e. we want last_page = "152", not "52"
        #if lastnum < self.first_page:
        #    len(lastnum)....
        try:
            lastnum = self.pages.split('-')[1]
            return lastnum
        except (IndexError, AttributeError):
            return None

    def _get_title(self):
        return self._get(self._root+'/Article/ArticleTitle') 

    def _get_volume(self):
        try:
            return self.content.find(self._root+'/Article/Journal/JournalIssue/Volume').text
        except AttributeError:
            return None

    def _get_issue(self):
        try:
            return self.content.find(self._root+'/Article/Journal/JournalIssue/Issue').text
        except AttributeError:
            return None

    def _get_volume_issue(self):
        ji = self.content.find(self._root+'/Article/Journal/JournalIssue')
        try:
            return '%s(%s)' % (ji.find('Volume').text, ji.find('Issue').text)
                               
        except AttributeError:
            pass
        try:
            return ji.find('Volume').text
        except AttributeError:
            pass
        # electronic pubs may not have volume or issue
        # e.g., http://www.ncbi.nlm.nih.gov/pubmed?term=20860988
        return None

    def _get_pubdate(self):
        return self._compose_datetime(self.content.find(self._root+'/Article/Journal/JournalIssue/PubDate'))

    def _get_year(self):
        y = self._get(self._root+'/Article/Journal/JournalIssue/PubDate/Year')
        if y is None:
            # case applicable for pmid:9887384 (at least)
            try:
                y = self._get(self._root+'/Article/Journal/JournalIssue/PubDate/MedlineDate')[0:4]
            except TypeError:
                pass
        return y

    def _get_doi(self):
        return self._get('PubmedData/ArticleIdList/ArticleId[@IdType="doi"]')

    def _get_pii(self):
        return self._get('PubmedData/ArticleIdList/ArticleId[@IdType="pii"]')

    def _get_pmc(self):
        try:
            return self._get('PubmedData/ArticleIdList/ArticleId[@IdType="pmc"]')[3:]
        except TypeError:
            return None

    def _get_issn(self):
        return self._get(self._root+'/Article/Journal/ISSN')

    def _get_mesh_headings(self):
        if self.pubmed_type=='book':
            return None

        meshtags = self.content.findall('MedlineCitation/MeshHeadingList/MeshHeading')
        outd = { }
        for mesh in meshtags:
            descript = mesh.find('DescriptorName')  # should always be present
            qual = mesh.find('QualifierName')       # may not be present

            DUI = descript.get('UI')
            outd[DUI] = { 
                    'descriptor_name': descript.text,
                    'major_topic': True if descript.get('MajorTopicYN')=='Y' else False,
                    'qualifier_name': None if qual is None else qual.text,
                    'qualifier_ui': None if qual is None else qual.get('UI'),
                }
        return outd

    def _get_chemicals(self):
        if self.pubmed_type=='book':
            return None

        outd = {}
        chemicals = self.content.findall('MedlineCitation/ChemicalList/Chemical')
        for chem in chemicals:
            substance = chem.find('NameOfSubstance')
            regnum = chem.find('RegistryNumber').text  # very often this is '0'
            outd[substance.get('UI')] = { 
                    'substance_name': substance.text,
                    'registry_number': regnum
                }
        return outd

    def _get_publication_types(self):
        outd = {}
        pubtypes = self.content.findall('MedlineCitation/Article/PublicationTypeList/PublicationType')
        for pt in pubtypes:
            outd[pt.get('UI')] = pt.text
        return outd
    
    def _get_grantlist(self):
        outl = []
        grants = self.content.findall('MedlineCitation/GrantList')
        for gr in grants:
            outl.append( {'agency': gr.get('Agency', None), 'country': gr.get('Country', None)} )
        return outl
        

    def __str__(self):
        if self.pubmed_type == 'article':
            return( '%s (%s. %s, %s:%s)' % (
                 self.title, self.authors_str, self.journal, self.volume_issue, self.pages) )
        else:
            return( '%s (%s. %s, %s)' % (self.title, self.authors_str, self.book_title, self.year))


############################################################################
## Utilities

def _au_to_last_fm(au):
    if au is None:
        return
    try:
        return au.find('LastName').text + u' ' + au.find('Initials').text
    except AttributeError:
        pass
    try:
        return au.find('CollectiveName').text 
    except AttributeError:
        pass
    try:
        return au.find('LastName').text
    except AttributeError:
        pass
    raise MetaPubError("Author structure not recognized")





import re
from urlparse import urlparse

# examples of real DOIs: 
#         10.1002/(SICI)1098-1004(1999)14:1<91::AID-HUMU21>3.0.CO;2-B
#         10.1007/s12020-014-0368-x
#         10.1101/gad.14.3.278
#         10.1016/S0898-1221(00)00204-2

# example of DOI returned by the regular expression that is incorrect:
#         10.1002/ajmg.b.30585).

re_doi = re.compile(r'(10[.][0-9]{2,}(?:[.][0-9]+)*/(?:(?!["&\'])\S)+)')
re_doi_ws = re.compile(r'(10[.][0-9]{2,}(?:[.][0-9]+)*\s+/\s+(?:(?!["&\'])\S)+)')

re_pmid = re.compile('\d+')
re_numbers = re_pmid    # for now, until there's a better idea about parsing PMIDs...

def pick_pmid(text):
    '''return longest numerical string from text (string) as the pmid.
        if text is empty or there are no pmids, return None.'''
    pmids = re_pmid.findall(text)
    if pmids:
        longest = ''
        for num in pmids:
            if len(num) > len(longest):
                longest = num
        return longest
    else:
        return None

def _doi_pass_2(doi):
    if doi.endswith('.') or doi.endswith(','):
        return _doi_pass_2(doi[:-1])
    elif doi.endswith(')'):
        if doi.count('(') == doi.count(')'):
            return doi
        else:
            return _doi_pass_2(doi[:-1])
    else:
        return doi.replace(' ', '')

def findall_dois_in_text(inp, whitespace=False):
    '''returns all seen DOIs in input string.

       if `whitespace` arg set to True, look for DOIs like the following:
             10.1002 / pd.354   
        ...but return with whitespace stripped:
             10.1002/pd.354
    '''
    if whitespace:
        dois = re_doi_ws.findall(inp)
    else:
        dois = re_doi.findall(inp)
    return [_doi_pass_2(doi) for doi in dois]

def find_doi_in_string(inp, whitespace=False):
    '''returns the first seen DOI in the input string.'''
    try:
        doi = findall_dois_in_text(inp, whitespace)[0]
    except IndexError:
        return None
    return _doi_pass_2(doi)

#### NOT TESTED and probably not working #####
def get_pmc_fulltext_filename_for_PubMedArticle(pma):
    fmt = '{journal}/{journal}_{year}_{month}_{day}_{voliss}_{pages}'
    return fmt.format(**pma.to_dict())

def get_nature_doi_from_link(link):
    '''custom method to get a DOI from a nature.com URL
    For example, http://www.nature.com/modpathol/journal/vaop/ncurrent/extref/modpathol2014160x3.xlsx
    :param link: the URL
    :return: a string containing a DOI, if one was resolved, or None
    '''
    if 'nature.com' not in link:
        return None

    # this is a non-comprehensive list of nature journals
    style1journals = ['gimo', 'nature', 'nbt', 'ncb', 'nchembio', 'ncomms', 'ng', 'nm', 'nn', 'nrc', 'nrm', 'nsmb', 'srep']

    # example: link:http://www.nature.com/modpathol/journal/vaop/ncurrent/extref/modpathol2014160x3.xlsx
    #          doi:10.1038/modpathol.2014.160
    style2journals = ['aps', 'bjc', 'cddis', 'cr', 'ejhg', 'gim', 'jcbfm', 'jhg', 'jid', 'labinvest', 'leu',
                      'modpathol', 'mp', 'onc', 'oncsis', 'pr']

    match = re.search(r'nature.com/[a-zA-z]+/', link)

    journal_abbrev = None
    if match:
        try:
            journal_abbrev = match.group(0).split('/')[1]
        except:
            log.debug("Unable to extract journal abbrev from link {}".format(link))
            journal_abbrev = None

    # Example: http://www.nature.com/neuro/journal/v13/n11/abs/nn.2662.html
    if journal_abbrev == 'neuro':
        journal_abbrev = 'nn'

    match = re.search(r'%s\.{0,1}\d+' % journal_abbrev, link)
    if match:
        doi_suffix = match.group(0)
        if doi_suffix.endswith('.'):  # strip off a trailing period
            doi_suffix = doi_suffix[:-1]

        # the DOI suffix can be taken directly for these journals
        if journal_abbrev in style1journals:
            return  '10.1038/{}'.format(doi_suffix)

        # style2journals are the default
        else:
            year = doi_suffix[len(journal_abbrev):len(journal_abbrev)+4]
            num = doi_suffix[len(journal_abbrev)+4:]
            return '10.1038/{}.{}.{}'.format(journal_abbrev, year, num)

    # http://www.nature.com/articles/cr2009141 :
    # http://www.nature.com/articles/cddis201475
    # http://www.nature.com/articles/nature03404
    # http://www.nature.com/articles/ng.2223
    # http://www.nature.com/articles/nsmb.2666
    match = re.search(r'articles/(([a-z]+)\.{0,1}(\d+))', link)
    if match:
        full_match = match.group(0)
        suffix = match.group(1)
        journal_abbrev = match.group(2)
        num = match.group(3)
        if journal_abbrev in style1journals:
            return '10.1038/{}'.format(suffix)
        else:
            return '10.1038/{}.{}.{}'.format(journal_abbrev, num[:4], num[4:])

    # http://www.nature.com/leu/journal/v19/n11/abs/2403943a.html : 10.1038/sj.leu.2403943
    # http://www.nature.com/onc/journal/v26/n57/full/1210594a.html :  doi:10.1038/sj.onc.1210594
    match = re.search(r'full/\d+|abs/\d+', link)
    if match:
        num = match.group(0).split('/')[1]
        return '10.1038/sj.{}.{}'.format(journal_abbrev, num)

def get_biomedcentral_doi_from_link(link):
    '''custom method to get a DOI from a biomedcentral.com URL
    For example, http://www.nature.com/modpathol/journal/vaop/ncurrent/extref/modpathol2014160x3.xlsx
    :param link: the URL
    :return: a string containing a DOI, if one was resolved, or None
    '''
    # style 1:
    # http://www.biomedcentral.com/content/pdf/bcr1282.pdf : doi:10.1186/bcr1282
    # http://www.biomedcentral.com/content/pdf/1465-9921-12-49.pdf : doi:10.1186/1465-9921-12-49
    # http://www.biomedcentral.com/content/pdf/1471-2164-16-S1-S3.pdf : doi:10.1186/1471-2164-16-S1-S3
    # http://www.biomedcentral.com/content/pdf/1753-6561-4-s2-o22.pdf : doi:10.1186/1753-6561-4-S2-O22
    # http://genomebiology.com/content/pdf/gb-2013-14-10-r108.pdf : doi:10.1186/gb-2013-14-10-r108
    # for supplementary, must remove the last 'S' part
    # http://www.biomedcentral.com/content/supplementary/bcr1865-S3.doc : doi:10.1186/bcr1865
    # http://www.biomedcentral.com/content/supplementary/bcr3584-S1.pdf : doi:10.1186/bcr3584
    # http://www.biomedcentral.com/content/supplementary/1471-2105-11-300-S1.PDF : doi:10.1186/1471-2105-11-300
    # http://www.biomedcentral.com/content/supplementary/1471-2164-12-343-S3.XLS : doi:10.1186/1471-2164-12-343
    # http://www.biomedcentral.com/content/supplementary/1471-2164-14-S3-S7-S1.xlsx : doi:10.1186/1471-2164-14-S3-S7
    # http://www.biomedcentral.com/content/supplementary/gb-2013-14-10-r108-S8.xlsx : doi:10.1186/gb-2013-14-10-r108
    # style 2:
    # http://www.biomedcentral.com/1471-2148/12/114 : doi:10.1186/1471-2164-12-114
    # http://www.biomedcentral.com/1471-2164/15/707/table/T2 : doi:10.1186/1471-2164-15-707
    # http://www.biomedcentral.com/1471-2164/14/S1/S11 doi:10.1186/1471-2164-14-S1-S11
    # http://www.biomedcentral.com/1471-230X/11/31 doi:10.1186/1471-230X-11-31

    # first, try to use the filename
    if '/content/' in link:
        filename = link.split('/')[-1]
        if '.' in filename:
            base = filename.split('.')[0]
            if '/pdf/' in link:
                return '10.1186/' + base
            elif '/supplementary/' in link:
                i1 = base.rfind('S')
                i2 = base.rfind('s')
                i = max(i1, i2)
                return '10.1186/' + base[:i-1]
    else:
        parse_result = urlparse(link)
        path = parse_result.path
        keywords = ['abstract', 'figure', 'table']
        for kw in keywords:
            if kw in path:
                i = path.find(kw)
                path = path[:i-1]
                break
        if path[-1] == '/':
            path = path[:-1]
        if path[0] == '/':
            path = path[1:]
        return '10.1186/' + path.replace('/', '-')






# TODO
#
#11980567: Clin. Sci. -- no URL because No URL format for Journal Clin Sci
#12035837: Can J Neurol Sci -- no URL because No URL format for Journal Can J Neurol Sci
#12036192: Clin. Nephrol. -- no URL because No URL format for Journal Clin Nephrol
#12037434: Eur. Neurol. -- no URL because No URL format for Journal Eur Neurol
#12055472: Retina (Philadelphia, Pa.) -- no URL because No URL format for Journal Retina (Philadelphia, Pa)
#12063046: Jpn. J. Ophthalmol. -- no URL because No URL format for Journal Jpn J Ophthalmol
#12063482: J. Am. Acad. Dermatol. -- no URL because No URL format for Journal J Am Acad Dermatol
#12056405: Genes Cells -- no URL because No URL format for Journal Genes Cells
#12065946: Fetal. Diagn. Ther. -- no URL because No URL format for Journal Fetal Diagn Ther
#12066726: Br J Anaesth -- no URL because No URL format for Journal Br J Anaesth
#12071635: J. Neuropathol. Exp. Neurol. -- no URL because No URL format for Journal J Neuropathol Exp Neurol
#12071824: Arch Dermatol -- no URL because No URL format for Journal Arch Dermatol
#12070551: Saudi Med J -- no URL because No URL format for Journal Saudi Med J
#12074273: Circ. J. -- no URL because No URL format for Journal Circ J
#12076704: Oral Oncol. -- no URL because No URL format for Journal Oral Oncol
#12080609: Rinsho Shinkeigaku -- no URL because No URL format for Journal Rinsho Shinkeigaku
#12162770: Arch. Otolaryngol. Head Neck Surg. -- no URL because No URL format for Journal Arch Otolaryngol Head Neck Surg
#12170759: Cancer Biol. Ther. -- no URL because No URL format for Journal Cancer Biol Ther
#12173465: Mol. Biol. (Mosk.) -- no URL because No URL format for Journal Mol Biol (Mosk)
#12173720: Mayo Clin. Proc. -- no URL because No URL format for Journal Mayo Clin Proc
#12202459: Endocr. Rev. -- no URL because No URL format for Journal Endocr Rev
#12199334: J. Pediatr. Endocrinol. Metab. -- no URL because No URL format for Journal J Pediatr Endocrinol Metab
#12199344: J. Pediatr. Endocrinol. Metab. -- no URL because No URL format for Journal J Pediatr Endocrinol Metab
#12087194: Genes Genet. Syst. -- no URL because No URL format for Journal Genes Genet Syst
#12064832: J. Rheumatol. -- no URL because No URL format for Journal J Rheumatol
#12049533: Arch Surg -- no URL because No URL format for Journal Arch Surg
#10458450: J. Urol. -- no URL because No URL format for Journal J Urol
#10458483: Tohoku J. Exp. Med. -- no URL because No URL format for Journal Tohoku J Exp Med
#10458491: Tohoku J. Exp. Med. -- no URL because No URL format for Journal Tohoku J Exp Med
#10470409: Andrologia -- no URL because No URL format for Journal Andrologia
#10474162: Dev. Dyn. -- no URL because No URL format for Journal Dev Dyn
#10520237: Ophthalmic Genet. -- no URL because No URL format for Journal Ophthalmic Genet
#10521250: Ophthalmic Genet. -- no URL because No URL format for Journal Ophthalmic Genet
#21199372: no URL because No URL format for Journal Basic Clin Pharmacol Toxicol
#15533574: no URL because No URL format for Journal Int J Pediatr Otorhinolaryngol
#17100396: no URL because No URL format for Journal J Med Assoc Thai
#17415575: no URL because No URL format for Journal Arch. Dermatol. Res.
#17415800: no URL because No URL format for Journal Mov. Disord.
#17416296: no URL because No URL format for Journal Arch. Med. Res.
#17143317: no URL because No URL format for Journal Nat Clin Pract Endocrinol Metab
#17145028: no URL because No URL format for Journal Med Clin (Barc)
#17145065: no URL because No URL format for Journal Mutat Res
#15452722: no URL because No URL format for Journal Graefes Arch Clin Exp Ophthalmol

# HARDER CASES:
#
#10413889: Medicina (B Aires) -- no URL because No URL format for Journal Medicina (B Aires)
#12027577: Seizure -- no URL because No URL format for Journal Seizure
#10415464: Ophthalmic Genet. -- no URL because No URL format for Journal Ophthalmic Genet
#12015062: Zhonghua Xue Ye Xue Za Zhi -- no URL because No URL format for Journal Zhonghua Xue Ye Xue Za Zhi
#12015065: Zhonghua Xue Ye Xue Za Zhi -- no URL because No URL format for Journal Zhonghua Xue Ye Xue Za Zhi
#12032748: Int. J. Obes. Relat. Metab. Disord. -- no URL because No URL format for Journal Int J Obes Relat Metab Disord


doi2step_journals = [ 'Nat Nanotechnol' ]


todo_journals = {
    'Pharmacol Rep': { 'example': 'http://www.ncbi.nlm.nih.gov/pubmed/?term=23238479[uid] --> www.if-pan.krakow.pl/pjp/pdf/2012/5_1234.pdf' },
    'Med Sci Monit': { 'example': 'http://www.medscimonit.com/download/index/idArt/869530' },
    'Asian Pac J Cancer Prev': { 'example': 'http://www.apocpcontrol.org/paper_file/issue_abs/Volume12_No7/1771-1776%20c%206.9%20Lei%20Zhong.pdf' },
    'Rev Esp Cardiol': { 'example': 'http://www.revespcardiol.org/en/linkresolver/articulo-resolver/13131646/' },
    'Ann Dermatol Venereol': { 'example': 'http://www.em-consulte.com/article/152959/alertePM' },
    'Mol Cells': { 'example': 'http://www.molcells.org/journal/view.html?year=2004&volume=18&number=2&spage=141 --> http://pdf.medrang.co.kr/KSMCB/2004/018/mac-18-2-141.pdf'},
    'Mol Vis': { 'example': 'http://www.molvis.org/molvis/v10/a45/ --> http://www.molvis.org/bin/pdf.cgi?Zheng,10,45'},
    'Singapore Med J': { 'example': 'http://www.sma.org.sg/smj/4708/4708cr4.pdf' },
    'Rev Port Cardiol': { 'example': '16335287: http://www.spc.pt/DL/RPC/artigos/74.pdf' },
    'World J Gastroenterol': { 'example': 'http://www.wjgnet.com/1007-9327/full/v11/i48/7690.htm --> http://www.wjgnet.com/1007-9327/pdf/v11/i48/7690.pdf' },
    'Genet Mol Res': { 'example': '24668667: http://www.geneticsmr.com/articles/2992 --> http://www.geneticsmr.com//year2014/vol13-1/pdf/gmr2764.pdf' },
    'Arq Bras Endocrinol Metabol': { 'example': '15611820: http://www.scielo.br/pdf/abem/v48n1/19521.pdf' },
    'Neoplasma': { 'example': '17319787: http://www.elis.sk/download_file.php?product_id=1006&session_id=skl2f3grcd19ebnie17u15a571' },
    'Clinics (Sao Paulo)': { 'example': '17823699: http://www.scielo.br/scielo.php?script=sci_arttext&pid=S1807-59322007000400003' },
    'Asian J Androl': {'example': '18097502: http://www.asiaandro.com/Abstract.asp?doi=10.1111/j.1745-7262.2008.00376.x' },
    'Anesthesiology': {'example': '18212565: http://dx.doi.org/10.1097/01.anes.0000299431.81267.3e --> html w/ <a id="pdfLink" data-article-url="THE_URL">' },
    'Nat Prod Commun': {'example': '19634325 (no direct link found yet) -- http://www.naturalproduct.us/' },
    'Oncotarget': {'example': '26008984 (pii = 3900) --> http://www.impactjournals.com/oncotarget/index.php?journal=oncotarget&page=article&op=view&path=3900' },
    }


format_templates = {
    'acs': 'http://pubs.acs.org/doi/pdf/{a.doi}',
    'akademii': 'http://www.akademiai.com/content/{a.pii}/fulltext.pdf',
    'ats': 'http://www.atsjournals.org/doi/pdf/{a.doi}',
    'informa': 'http://informahealthcare.com/doi/pdf/{a.doi}',
    'lancet': 'http://www.thelancet.com/pdfs/journals/{ja}/PII{a.pii}.pdf',
    'liebert': 'http://online.liebertpub.com/doi/pdf/{a.doi}',
    'plos': 'http://www.plosbiology.org/article/fetchObjectAttachment.action?url=info:doi/{a.doi}&representation=PDF',
    'wiley': 'http://onlinelibrary.wiley.com/doi/{a.doi}/pdf',
    }

lancet_journals = {
    'Lancet': { 'ja': 'lancet' },
    'Lancet Diabetes Endocrinol': { 'ja': 'landia' },
    'Lancet Glob Health': { 'ja': 'langlo' },
    'Lancet Haematol': { 'ja': 'lanhae' },
    'Lancet HIV': { 'ja': 'lanhiv' },
    'Lancet Infect Dis': { 'ja': 'laninf' },
    'Lancet Neurol': { 'ja': 'laneur' },
    'Lancet Oncol': { 'ja': 'lanonc' },
    'Lancet Psychiatry': { 'ja': 'lanpsy' },
    'Lancet Respir Med': { 'ja': 'lanres' },
    }

#EBioMedicine


# the SD journals are also represented in simple_formats_pii
sciencedirect_journals = (
    'Ann Genet',
    'Arch Pediatr',
    'Blood Cells Mol Dis',
    'Biochem Biophys Res Commun' ,
    'Biochem Pharmacol',
    'Biochim Biophys Acta',
    'Clin Chim Acta',
    'Clin Immunol',
    'FEBS Lett',
    'Eur J Cancer',
    'Eur J Med Genet',
    'Gene',
    'Genomics',
    'Gynecol Obstet Fertil',
    'Hepatol Res',
    'Hum Immunol',
    'J Am Coll Cardiol',
    'J Mol Biol',
    'J Neurol Sci',
    'Life Sci',
    'Mol Cell Endocrinol',
    'Mol Cell Probes',
    'Mol Genet Metab',
    'Mol Immunol',
    'Mutat Res',
    'Neuromuscul Disord',
    'Neurosci Lett',
    'Pediatr Pulmonol',
    )


jama_journals = (
     'Arch Gen Psychiatry',
     'Arch Neurol',
     'Arch Ophthalmol',
     'JAMA',
     'JAMA Dermatol',
     'JAMA Facial Plast Surg',
     'JAMA Intern Med',
     'JAMA Neurol',
     'JAMA Oncol',
     'JAMA Ophthalmol',
     'JAMA Otolaryngol Head Neck Surg',
     'JAMA Pediatr',
     'JAMA Psychiatry',
     'JAMA Surg',
     )

# TODO
# doiserbia (Library of Serbia) articles can be grabbed by doing the_doi_2step,
# then ...?
doiserbia_journals = ['Genetika']

# JSTAGE: mostly free (yay)
# Examples:
# J Biochem: https://www.jstage.jst.go.jp/article/biochemistry1922/125/4/125_4_803/_pdf
# Drug Metab Pharmacokinet: https://www.jstage.jst.go.jp/article/dmpk/20/2/20_2_144/_article --> https://www.jstage.jst.go.jp/article/dmpk/20/2/20_2_144/_pdf
jstage_journals = [
    'J Periodontol',
    'J Biochem',
    'Drug Metab Pharmacokinet',
    'Endocr J',
    ]

# cell journals
#cell_format = 'http://download.cell.com{ja}/pdf/PII{pii}.pdf'
cell_format = 'http://www.cell.com{ja}/pdf/{pii}.pdf'
cell_journals = {
    'Am J Hum Genet': { 'ja': '/AJHG' },
    'Cancer Cell': { 'ja': '/cancer-cell' },
    'Cell': { 'ja': '' },
    'Cell Host Microbe': {'ja': '/cell-host-microbe' },
    'Cell Metab': {'ja': '/cell-metabolism' },
    'Cell Stem Cell': {'ja': '/cell-stem-cell' },
    'Chem Biol': {'ja': '/chemistry-biology' },
    'Curr Biol': {'ja': '/current-biology' },
    'Dev Cell': {'ja': '/developmental-cell' },
    'Immunity': {'ja': '/immunity' },
    'Mol Cell': {'ja': '/molecular-cell' },
    'Neuron': {'ja': '/neuron' },
    'Structure': {'ja': '/structure' },
    'Trends Mol Med': { 'ja': '/trends' },
    }

# nature journals -- COMPLETE
nature_format = 'http://www.nature.com/{ja}/journal/v{a.volume}/n{a.issue}/pdf/{a.pii}.pdf'
nature_journals = {
    'Eur J Hum Genet': { 'ja': 'ejhg' },
    'Eye (Lond)': { 'ja': 'eye' },
    'Genes Immun': { 'ja': 'gene' },
    'Genet Med': { 'ja': 'gim' },
    'J Invest Dermatol': { 'ja': 'jid' },
    'J Hum Genet': { 'ja': 'jhg' },
    'Kidney Int': { 'ja': 'ki' },
    'Leukemia': { 'ja': 'leu' },
    'Mod Pathol': { 'ja': 'modpathol' },
    'Mol Psychiatry': { 'ja': 'mp' },
    'Nature': { 'ja': 'nature' },
    'Nat Chem': { 'ja': 'nchem' },
    'Nat Clin Pract Endocrinol Metab': { 'ja': 'nrendo' },
    'Nat Clin Pract Cardiovasc Med': { 'ja': 'nrcardio' },
    'Nat Clin Pract Oncol': { 'ja': 'nrclinonc' },
    'Nat Clin Pract Gastroenterol Hepatol': { 'ja': 'nrgastro' },
    'Nat Clin Pract Urol': { 'ja': 'nrurol' },
    'Nat Clin Pract Neurol': { 'ja': 'nrneurol' },
    'Nat Clin Pract Nephrol': { 'ja': 'nrneph' },
    'Nat Clin Pract Rheumatol': { 'ja': 'nrrheum' },
    'Nat Genet': { 'ja': 'ng' },
    'Nat Commun': {'ja': 'ncomms' },
    'Nat Neurosci': { 'ja': 'neuro' },
    'Nat Mater': { 'ja': 'nmat' },
    'Nat Med': { 'ja': 'nm' },
    'Nat Methods': { 'ja': 'nmeth' },
    'Nat Protoc': { 'ja': 'nprot' },
    'Nat Rev Drug Discov': { 'ja': 'nrd' },
    'Nat Rev Cardiol': { 'ja': 'nrcardio' },
    'Nat Rev Clin Oncol': { 'ja': 'nrclinonc' },
    'Nat Rev Endocrinol': { 'ja': 'nrendo' },
    'Nat Rev Genet': { 'ja': 'nrg' },
    'Nat Rev Gastroenterol Hepatol': { 'ja': 'nrgastro' },
    'Nat Rev Nephrol': { 'ja': 'nrneph' },
    'Nat Rev Neurol': { 'ja': 'nrneurol' },
    'Nat Rev Rheumatol': { 'ja': 'nrrheum' },
    'Nat Rev Urol': { 'ja': 'nrurol' },
    'Nature reviews Immunology': { 'ja': 'nri' },
    'Neuropsychopharmacology': { 'ja': 'npp' },
    'Oncogene': { 'ja': 'onc' },
    'Pediatr Res': { 'ja': 'pr' },
    }



# simple formats are used for URLs that can be deduced from PubMedArticle XML
simple_formats_doi = {
    'Acta Oncol': format_templates['informa'],
    'Hemoglobin': format_templates['informa'],
    'Platelets': format_templates['informa'],
    'Xenobiotica': format_templates['informa'],

    'Am J Respir Cell Mol Biol': format_templates['ats'],
    'Am J Respir Crit Care Med': format_templates['ats'],

    'Anal Chem': format_templates['acs'],
    'Biochemistry': format_templates['acs'],

    'Genet Test': format_templates['liebert'],
    'Genet Test Mol Biomarkers': format_templates['liebert'],
    'Thyroid': format_templates['liebert'],

    'Endocrinology': 'http://press.endocrine.org/doi/pdf/{a.doi}',
    'Mol Endocrinol': 'http://press.endocrine.org/doi/pdf/{a.doi}',
    'J Periodontol': 'http://www.joponline.org/doi/pdf/{a.doi}',

    'PLoS Biol': format_templates['plos'],
    'PLoS Comput Biol': format_templates['plos'],
    'PLoS Genet': format_templates['plos'],
    'PLoS Med': format_templates['plos'],
    'PLoS ONE': format_templates['plos'],
    'PLoS Pathog': format_templates['plos'],
    'N Engl J Med':  'http://www.nejm.org/doi/pdf/{a.doi}',
    }



simple_formats_pii = {
    'Am Heart J': 'http://www.ahjonline.com/article/{a.pii}/pdf', #ScienceDirect
    'Am J Cardiol': 'http://www.ajconline.org/article/{a.pii}/pdf',
    'Am J Ophthalmol': 'http://www.ajo.com/article/{a.pii}/pdf', #ScienceDirect
    'Am J Med': 'http://www.amjmed.com/article/{a.pii}/pdf', #ScienceDirect
    'Atherosclerosis': 'http://www.atherosclerosis-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Arch Med Res': 'http://www.arcmedres.com/article/{a.pii}/pdf', #ScienceDirect
    'Biol Psychiatry': 'http://www.biologicalpsychiatryjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Bone': 'http://www.thebonejournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Brain Dev': 'http://www.brainanddevelopment.com/article/{a.pii}/pdf', #ScienceDirect
    'Cancer Cell Int': 'http://www.cancerci.com/content/pdf/{a.pii}.pdf', #Biomed Central
    'Cancer Genet Cytogenet': 'http://www.cancergeneticsjournal.org/article/{a.pii}/pdf', #ScienceDirect
    'Cancer Lett': 'http://www.cancerletters.info/article/{a.pii}/pdf', #ScienceDirect
    'Clin Neurol Neurosurg': 'http://www.clineu-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Diabetes Res Clin Pract': 'http://www.diabetesresearchclinicalpractice.com/article/{a.pii}/pdf', #ScienceDirect
    'Diagn Pathol': 'http://www.diagnosticpathology.org/content/pdf/{a.pii}.pdf', # Biomed Central
    'Epilepsy Res': 'http://www.epires-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Eur J Paediatr Neurol': 'http://www.ejpn-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Exp Hematol': 'http://www.exphem.org/article/{a.pii}/pdf', #ScienceDirect
    'Fertil Steril': 'http://www.fertstert.org/article/{a.pii}/pdf',    #ScienceDirect
    'Gastroenterology': 'http://www.gastrojournal.org/article/{a.pii}/pdf',
    'Gynecol Oncol': 'http://www.gynecologiconcology-online.net/article/{a.pii}/pdf', #ScienceDirect
    'Heart Rhythm': 'http://www.heartrhythmjournal.com/article/{a.pii}/pdf',
    'Int J Pediatr Otorhinolaryngol': 'http://www.ijporlonline.com/article/{a.pii}/pdf', #ScienceDirect
    'Int J Cardiol': 'http://www.internationaljournalofcardiology.com/article/{a.pii}/pdf', #ScienceDirect
    'J Dermatol': 'http://www.jdsjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'J Mol Cell Cardiol': 'http://www.jmmc-online.com/article/{a.pii}/pdf', #ScienceDirect
    'J Mol Diagn': 'http://jmd.amjpathol.org/article/{a.pii}/pdf',  #ScienceDirect
    'J Neurol Sci': 'http://www.jns-journal.com/article/{a.pii}/pdf',
    'J Pediatr': 'http://www.jpeds.com/article/{a.pii}/pdf',  #ScienceDirect
    'J Pediatr Urol': 'http://www.jpurol.com/article/{a.pii}/pdf',  #ScienceDirect
    'Ophthalmology': 'http://www.aaojournal.org/article/{a.pii}/pdf', #ScienceDirect
    'Orv Hetil': format_templates['akademii'],
    'Med Hypotheses': 'http://www.medical-hypotheses.com/article/{a.pii}/pdf', #ScienceDirect
    'Metabolism': 'http://www.metabolismjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Metab Clin Exp': 'http://www.metabolismjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Neurobiol Aging': 'http://www.neurobiologyofaging.org/article/{a.pii}/pdf', #ScienceDirect
    'Neuromuscul Disord': 'http://www.nmd-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Parkinsonism Relat Disord': 'http://www.prd-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Pediatr Neurol': 'http://www.pedneur.com/article/{a.pii}/pdf', #ScienceDirect
    'Placenta': 'http://www.placentajournal.org/article/{a.pii}/pdf', #ScienceDirect
    'Surg Neurol': 'http://www.worldneurosurgery.org/article/{a.pii}/pdf', #ScienceDirect
    'Thromb Res': 'http://www.thrombosisresearch.com/article/{a.pii}/pdf', #ScienceDirect
    }

#    The following were taken out of simple_formats_pii but kept in the sciencedirect_journals list:
#    'Mol Genet Metab': 'http://www.mgmjournal.com/article/{a.pii}/pdf', #ScienceDirect


# Many BMC journals start with "BMC" (they're covered automatically) --
#   this list covers the ones that don't.  #TODO: Gather up the ones not in here yet.
BMC_journals = (
    'Diagn Pathol',
    'Genome Biol',
    'Genome Med',
    'Hum Genomics',
    'J Clin Bioinforma',
    'J Transl Med',
    )

# the "aid" is the second half of the DOI string (after the slash)
BMC_format = 'http://www.biomedcentral.com/content/pdf/{aid}.pdf'

# vip = Volume-Issue-Page format 
#       URLs that have the same format except for the host name

vip_format = 'http://{host}/content/{a.volume}/{a.issue}/{a.first_page}.full.pdf'

vip_journals = {
        'Ann Clin Biochem': { 'host': 'acb.sagepub.com' },
        'Am J Clin Pathol': { 'host': 'ajcp.ascpjournals.org' },
        'Am J Hypertens': { 'host': 'ajh.oxfordjournals.org' },
        'Ann Oncol' : {'host' : 'annonc.oxfordjournals.org'},
        'Arterioscler Thromb Vasc Biol' : {'host' : 'atvb.ahajournals.org'},
        #TODO: real url is http://www.bloodjournal.org/content/bloodjournal/122/23/3844.full.pdf
        'Blood': { 'host': 'bloodjournal.org' },
        'Brain': { 'host': 'brain.oxfordjournals.org' },
        'Breast Cancer Res' : { 'host': 'breast-cancer-research.com' },
        'Cancer Discov' : {'host': 'cancerdiscovery.aacrjournals.org'},
        'Cancer Epidemiol Biomarkers Prev': { 'host': 'cebp.aacrjournals.org' },
        'Cancer Res': { 'host': 'cancerres.aacrjournals.org' },
        'Carcinogenesis': { 'host': 'carcin.oxfordjournals.org' } ,
        'Cardiovasc Res' : {'host' : 'cardiovascres.oxfordjournals.org'},
        'Circulation': { 'host': 'circ.ahajournals.org' },
        'Circ Arrhythm Electrophysiol': { 'host': 'circep.ahajournals.org' },
        'Circ Cardiovasc Genet' : {'host' : 'circgenetics.ahajournals.org'},
        'Circ Res' : {'host' : 'circres.ahajournals.org'},
        'Clin Cancer Res' : {'host' : 'clincancerres.aacrjournals.org'},
        'Clin Chem' : {'host' : 'clinchem.org'},
        'Clin Infect Dis': { 'host': 'cid.oxfordjournals.org'},
        'Diabetes': {'host': 'diabetes.diabetesjournals.org'},
        'Diabetes Care': { 'host': 'care.diabetesjournals.org' },
        'Drug Metab Dispos': { 'host': 'dmd.aspetjournals.org' },
        'Endocr Relat Cancer': { 'host': 'erc.endocrinology-journals.org' },
        'Eur Heart J' : {'host' : 'eurheartj.oxfordjournals.org'},
        'Eur J Endocrinol' : {'host' : 'eje-online.org'},
        'FASEB J' : {'host' : 'fasebj.org'},
        'Genome Biol' : { 'host' : 'genomebiology.com'},
        'Genome Res': { 'host': 'genome.cshlp.org'},
        'Genes Dev' : {'host': 'genesdev.cshlp.org'},
        'Gut' : {'host' : 'gut.bmj.com'},
        'Haematologica' : {'host' : 'haematologica.org'},
        'Hum Mol Genet': { 'host': 'hmg.oxfordjournals.org' },
        'Hum Reprod': { 'host': 'humrep.oxfordjournals.org' },
        'Hypertension': { 'host': 'hyper.ahajournals.org' },
        'Invest Ophthalmol Vis Sci': { 'host': 'www.iovs.org' },
        'IOVS' : {'host' : 'iovs.org'},
        'J Am Soc Nephrol' : {'host' : 'jasn.asnjournals.org'},
        'J Biol Chem': { 'host': 'www.jbc.org' },
        'J Cell Biol' : {'host' : 'jcb.rupress.org'},
        'J Cell Sci' : {'host' : 'jcs.biologists.org'},
        'J Child Neurol': { 'host': 'jcn.sagepub.com'},
        'J Clin Endocrinol Metab': { 'host': 'jcem.endojournals.org' },
        'J Clin Oncol': { 'host': 'jco.ascopubs.org' },
        'J Dent Res': { 'host': 'jdr.sagepub.com' },
        'J Gerontol A Biol Sci Med Sci': { 'host': 'biomedgerontology.oxfordjournals.org' },
        'J Immunol' : {'host' : 'jimmunol.org'},
        'J Infect Dis': {'host': 'jid.oxfordjournals.org'},
        'J Natl Cancer Inst': {'host': 'jnci.oxfordjournals.org'},
        'J Neurol Neurosurg Psychiatry': {'host': 'jnnp.bmj.com'},
        'J Neurosci' : {'host' : 'jneurosci.org'},
        'J Med Genet': { 'host': 'jmg.bmj.com' },
        'J Mol Endocrinol': { 'host': 'jme.endocrinology-journals.org' },
        'J Lipid Res': { 'host': 'www.jlr.org' },
        'J Pharmacol Exp Ther': { 'host': 'jpet.aspetjournals.org' },
        'Mol Biol Cell' : {'host' : 'molbiolcell.org'},
        'Mol Cell Biol' : {'host': 'mcb.asm.org'},
        'Mol Canc Therapeut' : {'host' : 'mct.aacrjournals.org'},
        'Mol Hum Reprod': {'host': 'molehr.oxfordjournals.org'},
        'Mol Pharmacol': {'host' : 'molpharm.aspetjournals.org'},
        'Neurology' : {'host' : 'neurology.org'},
        'Nephrol Dial Transplant': {'host': 'ndt.oxfordjournals.org'},
        'Nucleic Acids Res' : {'host' : 'nar.oxfordjournals.org'},
        'Orphanet J Rare Dis' : {'host' : 'ojrd.com'},
        'Pediatrics': {'host': 'pediatrics.aappublications.org'},
        'Proc Natl Acad Sci USA': { 'host': 'pnas.org'},
        'QJM': { 'host': 'qjmed.oxfordjournals.org'},
        'Science': { 'host': 'sciencemag.org' },
        'Thorax': { 'host': 'thorax.bmj.com' },
        }

# Science (AAAS) -- requires login "as a courtesy to the community".  Mkay.
aaas_format = 'http://{ja}.sciencemag.org/{a.volume}/{a.issue}/{a.first_page}.full.pdf'
aaas_journals = {
        'Sci Signal': { 'ja': 'stke' },
        'Sci Transl Med': { 'ja': 'stm' },
    }


# Spandidos: volume/issue/firstpage AND a journal abbreviation. Fancy.
spandidos_format = 'http://www.spandidos-publications.com/{ja}/{a.volume}/{a.issue}/{a.first_page}/download'
spandidos_journals = {
        'Int J Oncol': { 'ja' : 'ijo' },
        'Int J Mol Med': { 'ja': 'ijmm' },
        'Oncol Lett': { 'ja' : 'ol' },
        'Oncol Rep': { 'ja' : 'or' },
    }


wiley_journals = (
    'Acta Neurol Scand',
    'Acta Ophthalmol Scand',
    'Ann Hum Genet',
    'Ann Neurol',
    'Am J Hematol',
    'Am J Med Genet',
    'Am J Med Genet A',
    'Am J Med Genet B Neuropsychiatr Genet',
    'Arthritis Rheum',
    'Australas J Dermatol',
    'Basic Clin Pharmacol Toxicol',
    'BJU Int',
    'Breast J',
    'Br J Dermatol',
    'Br J Haematol',
    'Cancer',
    'Clin Endocrinol (Oxf)',
    'Clin Exp Dermatol',
    'Clin Genet',
    'Clin Pharmacol Ther',
    'Diabet Med',
    'Dev Med Child Neurol',
    'Electrophoresis',
    'Environ Mol Mutagen',
    'Eur J Clin Invest',
    'Eur J Haematol',
    'Eur J Neurol',
    'Exp Dermatol',
    'Genes Chromosomes Cancer',
    'Genet Epidemiol',
    'Haemophilia',
    'Head Neck',
    'Hum Mutat',
    'Immunol Rev',
    'Int J Cancer',
    'Int J Lab Hematol',
    'J Bone Miner Res',
    'J Dermatol',
    'J Eur Acad Dermatol Venereol',
    'J Gastroenterol Hepatol',
    'J Intern Med',
    'J Orthop Res',
    'J Thromb Haemost',
    'J Pathol',
    'J Viral Hepat',
    'Mol Carcinog',
    'Mov Disord',
    'Muscle Nerve',
    'Neuropathol Appl Neurobiol',
    'Pediatr Blood Cancer',
    'Pediatr Int',
    'Prenat Diagn',
    'Proteins',
    'Scand J Immunol',
    'Tissue Antigens',
    'Transfus Med',
    'Transfusion',
    'Vox Sang',
    )


# Below: Journals with really annoying paywalls guarding their precious secrets.
schattauer_journals = [
    'Thromb Haemost',
    ]

# Royal Society of Chemistry
RSC_journals = [ 'Nat Prod Rep' ]

wolterskluwer_journals = [
    'AIDS',
    'Blood Coagul Fibrinolysis',
    'Clin Dysmorphol',
    'Curr Opin Hematol',
    'Eur J Gastroenterol Hepatol',
    'J Hypertens',
    'J Glaucoma',
    'J Pediatr Hematol Oncol',
    'J Pediatr Gastroenterol Nutr',
    'Medicine (Baltimore)',
    'Neuroreport',
    'Obstet Gynecol',
    'Pediatr Infect Dis J',
    'Pharmacogenet Genomics',
    'Pharmacogenetics',
    'Plast Reconstr Surg',
    'Psychiatr Genet',
    ]

karger_journals = [
    'Acta Haematol',
    'Cell Physiol Biochem',
    'Cytogenet Genome Res',
    'Dermatology (Basel)',
    'Horm Res',
    'Hum Hered',
    'Nephron',
    'Nephron Physiol',
    ]

springer_journals = [
    'Acta Neuropathol',
    'Ann Hematol',
    'Ann Surg Oncol',
    'Arch Dermatol Res',
    'Biochem Genet',
    'Breast Cancer Res Treat',
    'Calcif Tissue Int',
    'Cell Mol Neurobiol',
    'Diabetologia',
    'Eur J Pediatr',
    'Fam Cancer',
    'Graefes Arch Clin Exp Ophthalmol',
    'HNO',
    'Hum Genet',
    'Immunogenetics',
    'Int J Colorectal Dis',
    'J Bone Miner Metab',
    'J Endocrinol Invest',
    'J Inherit Metab Dis',
    'J Neurol',
    'J Mol Med',
    'J Mol Med (Berl)',
    'J Mol Neurosci',
    'Mod Rheumatol',
    'Neurogenetics',
    'Ophthalmologe',
    'Pediatr Nephrol',
    'Physiol Genomics',
    'Rheumatol Int',
    'World J Surg',
    ]

# thieme journals so far don't seem to have any open access content.
# example links to article page: https://www.thieme-connect.com/DOI/DOI?10.1055/s-0028-1085467
#           https://www.thieme-connect.com/DOI/DOI?10.1055/s-2007-1004566
thieme_journals = ['Neuropediatrics', 'Semin Vasc Med', 'Exp Clin Endocrinol Diabetes']

paywall_journals = schattauer_journals + wolterskluwer_journals + springer_journals + thieme_journals + karger_journals


#All in PMC (no need to write formats for):
 #Nat Comput
 #Nat Clim Chang
 #Nat Geosci
 #Nat Resour Model
 #Nat Lang Linguist Theory
 #Nat Photonics
 #Nat Phys
 #Nat Prod Bioprospect
 #Nat Lang Eng
 #Nat Rep Stem Cells
 #Nat Sci (Irvine)
 #Nat Sci Sleep



__doc__='currently-experimental mashups of various services to get needed IDs.'

from .text_mining import re_doi

def _assert_match_is_string_length(match, inp):
    if match:
        len_match = match.end() - match.start()
        if len_match == len(inp):
            assert True
        else:
            assert False
    else:
        assert False

def assert_is_good_doi(doi):
    match = re_doi.match(doi)
    _assert_match_is_string_length(match, doi)

def assert_is_good_pmid(pmid):
    match = re_pmid.match(pmid)
    _assert_match_is_string_length(match, doi)




from __future__ import absolute_import

import os
from lxml import etree
from urllib import urlretrieve

from .config import PKGNAME, DEFAULT_EMAIL, TMPDIR

PMC_ID_CONVERSION_URI = 'http://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool='+PKGNAME+'&email='+DEFAULT_EMAIL+'&ids=%s'

__doc__='''An assortment of functions providing access to various web APIs.

    The pubmedcentral.* functions abstract the submission of one of the following
    acceptable IDs to the Pubmed Central ID Conversion API as a lookup to
    get another ID mapping to the same pubmed article:

        * doi       Digital Object Identifier
        * pmid      Pubmed ID
        * pmcid     Pubmed Central ID (includes Versioned Identifier)

    Available functions:

        get_pmid_for_otherid(string)
    
        get_doi_for_otherid(string)

        get_pmcid_for_otherid(string)
'''

def get_tmp_xml_path(someid):
    someid = someid.replace('/', '__')
    return os.path.join(TMPDIR, '%s.xml' % someid)

def _pmc_id_conversion_api(input_id):
    xmlfile = urlretrieve(PMC_ID_CONVERSION_URI % input_id, get_tmp_xml_path(input_id))
    root = etree.parse(xmlfile[0])
    root.find('record')
    record = root.find('record')
    return record

def get_pmid_for_otherid(otherid):
    '''use the PMC ID conversion API to attempt to convert either PMCID or DOI to a PMID.
        returns PMID if successful, or None if there is no 'pmid' item in the return.

        :param: otherid (string)
        :rtype: string
    '''
    record = _pmc_id_conversion_api(otherid)
    return record.get('pmid')

def get_pmcid_for_otherid(otherid):
    record = _pmc_id_conversion_api(otherid)
    return record.get('pmcid')

def get_doi_for_otherid(otherid):
    record = _pmc_id_conversion_api(otherid)
    return record.get('doi')


# PMID: http://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool=my_tool&email=my_email@example.com&ids=23193287
# PMCID: http://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool=my_tool&email=my_email@example.com&ids=PMC3531190
# Manuscript ID: http://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool=my_tool&email=my_email@example.com&ids=NIHMS311352
# DOI: http://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool=my_tool&email=my_email@example.com&ids=10.1093/nar/gks1195
# Versioned identifier: http://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool=my_tool&email=my_email@example.com&ids=PMC2808187.1


